/*********************************************************************************
*         Beispiel Ansteuerung SAA1064
*         ============================
*                   Uwe Berger; 2009
*
* Es wird eine kleine Animation auf der 7-Segment-Anzeige ausgegeben.
*
*  ---------
*  Have fun!
*
**********************************************************************************/
#ifndef F_CPU
	#define F_CPU 12000000
#endif

#include <util/delay.h>
#include <avr/pgmspace.h>

#include "i2cmaster.h"

#define ADDR_7SEGM 	0x70

// eventuell an eigene Verdrahtung anpassen (SAA1064 -> 7-Segment-Anz.)
// Bit:		7	6	5	4	3	2	1	0
// Segment:	p	g	f	e	d	c	b	a
//
//  -a-
// |f  |b
//  -g-
// |e  |c
//  -d- .p
const uint8_t intro[] PROGMEM = {
	0b00000000,				// 	
	0b00000000,				// 	
	0b00000000,				// 	
	0b00000000,				// 	
	0b01111100,				// B	
	0b00111000,				// L
	0b00010000,				// I
	0b01111000,				// T
	0b01011011,				// 2
	0b00111111,				// 0	
	0b00111111,				// 0	
	0b01111111,				// 8
	0b01000000,				// -
	0b01111100,				// B
	0b01011100,				// O
	0b01110111,				// A
	0b01010000,				// R
	0b01011110,				// D
	0b00000000,				// 	
	0b00000000,				// 	
	0b00000000,				// 	
	0b00000000				// 	
};

//************************************************************************
void my_delay_ms(uint16_t ms)
{
	uint16_t i;
	for (i=0; i<ms; i++) _delay_ms(1);
}

//************************************************************************
void clear_all_digits(void)
{
	uint8_t i;
	i2c_start(ADDR_7SEGM+I2C_WRITE);
	i2c_write(1);						// auf Adresse 1 "positionieren"
	for (i=1; i<5; i++)	i2c_write(0);	// alle Segmente ausschalten
}

//************************************************************************
void saa1064_init(void)
{
	// SAA1064 Control-Register (Adresse 0)
    // Bit	7	6	5	4	3	2	1	0
	//		|	|	|	|	|	|	|	|
	//		|	|	|	|	|	|	|	- 0: Digit 1,2; 1: Digit 1...4
	//		|	|	|	|	|	|	----- 0/1: Digit 1,3 aus/an
	//		|	|	|	|	|	--------- 0/1: Digit 2,4 aus/an
	//		|	|	|	|	------------- 1: Testmode, alle Segmente an
	//		|	|	|	----------------- 1: plus 3mA
	//		|	|	--------------------- 1: plus 6mA
	//		|	------------------------- 1: plus 12mA
	//		----------------------------- nicht belegt
	i2c_start(ADDR_7SEGM+I2C_WRITE);
	// Controlregister setzen
	i2c_write(0);
	i2c_write(0b01110111);	// Digit 1...4, alle anwaehlbar, 21mA
	clear_all_digits();		// Anzeige loeschen
}

//************************************************************************
// Parameter: 
//     d  -> Digit (1...4)
//     v  -> anzuzeigender Wert (Zahlen 0...9, "-"; -> array digit[]
void write_digit(uint8_t d, uint8_t v)
{
	i2c_start(ADDR_7SEGM+I2C_WRITE);
	i2c_write(d);
	i2c_write(v);
}

//************************************************************************
//************************************************************************
//************************************************************************
int main(void)
{
	uint8_t i, j;

	i2c_init();								// TWI initialisieren
	saa1064_init();							// SAA1064 initialisieren

	while (1){
		// Intro-Text vertikal uebers Display schieben...
		for (i=0; i<(sizeof(intro)-3); i++) {
			for (j=1; j<5; j++) write_digit(j, pgm_read_byte(&intro[i+j-1]));
			my_delay_ms(400);
		}
	}					
}
